## Twisted Life: A game that uses a variant John H. Conway's Life cellular
## automaton in which Life forms twist around the center of the universe.
## Copyright (C) 2007  Paul Reiners
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random

class Life:
    def __init__(self, size = 17, universe = None):
        self.neighbors = ([[-1, -1], [-1, 0], [-1, 1],
                           [0, -1],           [0, 1],
                           [1, -1],  [1, 0],  [1, 1]])
        if universe == None:
            self.universe = []
            for i in range(size):
                row = []
                for j in range(size):
                    row.append(random.randint(0, 1))
                self.universe.append(row)
        else:
            self.universe = universe

    def __repr__(self):
        result = ''
        for i in range(len(self.universe)):
            for j in range(len(self.universe[i])):
                result += str(self.universe[i][j])
            result += '\n'

        return result

    def nextGen(self):
        newGen = []
        height = len(self.universe)
        width = len(self.universe[0])
        for i in range(height):
            newRow = []
            for j in range(width):
                count = 0
                for k in range(len(self.neighbors)):
                    neighbor = self.neighbors[k]
                    y = (i + neighbor[0]) % height
                    x = (j + neighbor[1]) % width
                    if self.universe[y][x]:
                        count += 1
                oldVal = self.universe[i][j]
                if count < 2 or count > 3:
                    newRow.append(0)
                elif oldVal == 0 and count == 3:
                    newRow.append(1)
                else:
                    newRow.append(oldVal)
            newGen.append(newRow)
        self.universe = newGen

    def isAlive(self, x, y):
        if self.universe[x][y]:
            return True
        else:
            return False

    def width(self):
        return len(self.universe[0])

    def height(self):
        return len(self.universe)

if __name__ == '__main__':
    universe = Life()
    print universe
    universe.nextGen()
    print universe
    universe.nextGen()
    print universe

    python =   Life(universe=[[0,0,0,0,0,0,0],
                 [0,0,0,0,0,0,0],
                 [0,0,0,0,1,1,0],
                 [0,1,0,1,0,1,0],
                 [0,1,1,0,0,0,0],
                 [0,0,0,0,0,0,0]])
    print python
    python.nextGen()
    print python
    python.nextGen()
    print python
    python.nextGen()
    print python
